/*
 * Resource.java
 * 
 * Created on 9-sep-2007, 15:10:28
 * 
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.sf.sql2java.common.resources;

import java.io.File;

import net.sf.sql2java.common.parameters.ParameterizedString;

/**
 *
 * @author dge2
 */
public class Resource extends ParameterizedString {
    public static final String DELIMITER = "*";
    
    private String raw = "";
    
// -- Constructors ------------------------------------------------------------
    
    public Resource(String init) {
        super(init);
    }
    
    public Resource() {
        super("");
    }
    
// -- Resource Manipulation ---------------------------------------------------
    
    /**
     * Add a part to the resource.
     * 
     * Later on, we'll parse the content to replace it with parameters.
     * 
     * @param part The part we want to add
     */
    public void add(String part) {
        // -- replace all '/' and '.' with the delimiter char
        part = part.replaceAll("/", DELIMITER);
        part = part.replaceAll("\\.", DELIMITER);
        
        // -- check if the part ends with a delimiter
        if ((content.length() > 0) && (! content.endsWith(DELIMITER)))
            this.content += DELIMITER;
        
        // -- add the part to the cache
        content += part;
    }
    
    /**
     * Add a raw part to the Resource content.
     * 
     * This part will not be parsed or searched for parameters.
     * 
     * @param part The part we want to add.
     */
    public void addRaw(String part) {
        // -- add the part to the cache
        raw += part;
    }
    
    /**
     * Add a parameter.
     * 
     * Add the value for the given parameter key. This key will be searched
     * inside the content of the resource and will be replaced by it's value
     * 
     * @param key	The parameter name
     * @param value	The parameter value
     */
    @Override
    public void addParameter(String key, String value) {
        super.addParameter(key, value);
    }
    
    /**
     * Clear the content of the resource
     */
    public void clear() {
        this.content = "";
        this.raw = "";
    }
    
// -- Getters -----------------------------------------------------------------
    
    /**
     * Get the resource as a package.
     * 
     * The content will be converted to a package String. This means all part
     * separators will be replaced by dots (.). Afterwards we'll parse the 
     * content to replace the parameter names with their values. Last thing
     * to do is adding the raw part.
     * 
     * @return the content of this resource, formed as a package
     */
    public String getPackage() {
        // -- create the result object
        String result = "";
        
        // -- replace the delimiter with "."
        content = content.replaceAll("\\" + DELIMITER, ".");
        
        // -- parse the result
        result = parse();
        
        // -- append raw data
        result += raw;
        
        // -- return the result
        return result;
    }
    
    /**
     * Get the resource as a path.
     * 
     * The content will be converted to a path String. This means all part
     * separators will be replaced by the systems file separator. Afterwards 
     * we'll parse the content to replace the parameter names with their 
     * values. Last thing to do is adding the raw part.
     * 
     * @return the content of this resource, formed as a path
     */
    public String getPath() {
        // -- create the result object
        String result = "";
        
        // -- replace the delimiter with "."
        content = content.replaceAll("\\" + DELIMITER, "\\" + File.separator);
        
        // -- parse the result
        result = parse();
        
        // -- append raw data
        result += raw;
        
        // -- return the result
        return result;
    }

    public String getRaw() {
    	return raw;
    }
    
// -- Setters -----------------------------------------------------------------
    
    public void setRaw(String raw) {
		this.raw = raw;
	}
}
