package net.sf.sql2java.data.holders;

import java.util.LinkedList;
import java.util.List;

import net.sf.sql2java.data.MapPointer;
import net.sf.sql2java.data.Path;
import net.sf.sql2java.data.node.BasicNodelet;

public class Table extends BasicNodelet {
	private static final long serialVersionUID = -3371655643759196893L;

	private String tableType;
	private String remarks;
	
// -- Constructors ------------------------------------------------------------
	
	public Table(String name) {
		super("table", name);
	}
	
// -- Columns -----------------------------------------------------------------
	
	public Column createColumn(String name) {
		// -- create the result
		Column result = new Column(name);
		
		// -- save the result
		saveColumn(result);
		
		// -- return the result
		return result;
	}
	
	public void saveColumn(Column column) {
		put(column);
	}
	
	public void removeColumn(String columnName) {
		String strPath = String.format(MapPointer.FORMAT, "column", columnName);
		
		remove(Path.createPath(strPath));
	}
	
	public Column getColumn(String columnName) {
		String strPath = String.format(MapPointer.FORMAT, "column", columnName);
		
		return get(Path.createPath(strPath));
	}
	
	public boolean hasColumn(String columnName) {
		String strPath = String.format(MapPointer.FORMAT, "column", columnName);
		
		return exists(Path.createPath(strPath));
	}
	
// -- Indices -----------------------------------------------------------------
	
	public Index createIndex(String name) {
		// -- create the result
		Index result = new Index(name);
		
		// -- save the result
		saveIndex(result);
		
		// -- return the result
		return result;
	}
	
	public void saveIndex(Index index) {
		put(index);
	}
	
	public void removeIndex(String indexName) {
		String strPath = String.format(MapPointer.FORMAT, "index", indexName);
		
		remove(Path.createPath(strPath));
	}
	
	public Index getIndex(String indexName) {
		String strPath = String.format(MapPointer.FORMAT, "index", indexName);
		
		return get(Path.createPath(strPath));
	}
	
	public boolean hasIndex(String indexName) {
		String strPath = String.format(MapPointer.FORMAT, "index", indexName);
		
		return exists(Path.createPath(strPath));
	}
	
// -- Getters -----------------------------------------------------------------

	public String getTableType() {
		return tableType;
	}
	
	public String getRemarks() {
		return remarks;
	}
	
	public List<Column> getColumns() {
		return list(Path.createPath("column"));
	}
	
	public List<Column> getPrimaryKeys() {
		List<Column> result = new LinkedList<Column>();
		
		for (Column col : getColumns())
			if (col.isPrimaryKey())
				result.add(col);
		
		return result;
	}
	
	public List<Index> getIndices() {
		return list(Path.createPath("index"));
	}
	
	public Schema getSchema() {
		if (getParent() == null) return null;
		else return (Schema) getParent();
	}
	
// -- Setters -----------------------------------------------------------------
	
	public void setTableType(String tableType) {
		this.tableType = tableType;
	}
	
	public void setRemarks(String remarks) {
		this.remarks = remarks;
	}
}
