/*
 * ScriptExecutor.java
 *
 * Created on 15 maart 2007, 14:01
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.generator.scripting;

import java.util.HashMap;
import java.util.Map;
import javax.script.Invocable;
import javax.script.ScriptEngine;
import javax.script.ScriptException;

import net.sf.sql2java.packages.TemplatePack;

/**
 *
 * @author annba
 */
public class ScriptEnvironment {
    private String version = "0.2 beta";

    private TemplatePack pack;
    
    private Map<String, Object> globals;
    
// -- Constructors ------------------------------------------------------------
    
    public ScriptEnvironment(TemplatePack pack) {
        this.pack = pack;
    }
    
// -- Script Execution --------------------------------------------------------
    
    public Object execute(String function) throws Exception {
        return executeReal(function);
    }
    
    public Object execute(String function, Object param1) throws Exception {
        return executeReal(function, param1);
    }
    
    public Object execute(String function, Object param1, Object param2) throws Exception {
        return executeReal(function, param1, param2);
    }
    
    public Object execute(String function, Object param1, Object param2, Object param3) throws Exception {
        return executeReal(function, param1, param2, param3);
    }
    
    public Object execute(String function, Object param1, Object param2, Object param3, Object param4) throws Exception {
        return executeReal(function, param1, param2, param3, param4);
    }
    
    public Object executeReal(String call, Object... attributes) throws Exception {
    	// -- create the result object
        Object result = null;
        
        // -- split the call into the scriptname and the functionname
        int index = call.indexOf(".");
        String scriptName = call.substring(0, index);
        String functionName = call.substring(index + 1);
        
        // -- Lets get the script
        Script script = pack.getScript(scriptName);
        
        // -- throw an exception if we can't find the script
        if (script == null)
            throw new IllegalArgumentException("The script '" + scriptName + "' could not be found!");
        
        try {
            // -- get the scripting engine
            ScriptEngine engine = script.getEngine();
            
            // -- fail if the script is invalid
            if (engine == null)
            	throw new ScriptException("Unable to load script engine '" + script.getEngineType() + "'");
            
            // -- initialize the scripting engine with the globals
            for (String key : globals.keySet())
                engine.put(key, globals.get(key));

            // -- load the script into the engine
            engine.eval(script.getContent());

            // -- execute the method 
            Invocable inv = (Invocable) engine;

            result = inv.invokeFunction(functionName, attributes);
        } catch (Exception e) {
            throw new Exception("There was a problem while executing method '" + scriptName + "." + functionName + "' --> " + e.getMessage(), e );
        }
        
        // -- return the result object
        return result;
    }
    
// -- Script Global Variables -------------------------------------------------
    
    public void addGlobal(String key, Object value) {
        if (globals == null)
            globals = new HashMap<String, Object>();
        
        globals.put(key, value);
    }
    
    public void addAllGlobals(Map<String, Object> toMerge) {
        if (globals == null)
            globals = new HashMap<String, Object>();
        
        globals.putAll(toMerge);
    }
    
// -- Getters -----------------------------------------------------------------
    
    public String getVersion() {
        return "Script Environment version " + version;
    }
    
// -- Setters -----------------------------------------------------------------

    public void setVersion(String version) {
        this.version = version;
    }
}
