/*
 * StringUtils.java
 *
 * Created on March 17, 2007, 4:17 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.sf.sql2java.utils;

/**
 *
 * @author dge2
 */
public abstract class StringUtils {
	public static final String PREFIX = ":\\{";
    public static final String SUFFIX = "\\}";

	public static String parameterize(String parameter) {
    	if (parameter == null)
    		return PREFIX + "(((#?)\\w)*)(\\@.{2})*" + SUFFIX;
    	else
    		return PREFIX + "((?i)" + parameter + ")(\\@.{2})*" + SUFFIX;
    }
	
    /**
     * Capitalize the first character of the input
     *
     */
    public static String capitalize(String input) {
        String result = "";
        
        if (input.length() > 0)
            result += input.substring(0, 1).toUpperCase();
        
        if (input.length() > 1)
            result += input.substring(1);
        
        return result;
    }
    
    public static String decapitalize(String input) {
        String result = "";
        
        if (input.length() > 0)
            result += input.substring(0, 1).toLowerCase();
        
        if (input.length() > 1)
            result += input.substring(1);
        
        return result;
    }
    
// -- Sql Specific String Conversions -----------------------------------------
    
    public static String toColumn(String input) {
        return input;
    }
    
    public static String toTable(String input) {
        return input;
    }
    
// -- Java Specific String Conversions ----------------------------------------
    
    public static String toVariable(String input) {
        return camelcase(input, false, true);
    }
    
    public static String toClass(String input) {
        return camelcase(input, true, true);
    }
    
    /**
     * Camelcase a String.
     *
     * If the String contains an '_' the next character will be converted to
     * uppercase. for example:<br/>
     *<br/>
     * my_text_example --> to myTextExample<br>
     *<br>
     * When the convertFirst flag is set, the first letter of the string will be
     * converted too:<br>
     *<br>
     * my_text_example --> MyTextExample<br>
     *<br>
     * Strings in uppercase might give some problems. By setting the 
     * lowercaseRest flag, we'll be able to create some nice camelcase Strings:<br>
     *<br>
     * My_TeXT_EXAMplE --> MyTextExample
     *
     * @param input
     * @param convertFirst  By setting this value we will be able to convert the 
     *                      first letter too.
     * @param lowercaseRest If lowercaseRest is set, we'll try to convert every 
     *                      non uppercase character to it's lowercase variant. 
     *                      This will create nice camelcase strings, even with 
     *                      capitalized ones
     *
     * @return  A nice camelcase'ed String
     */
    public static String camelcase(String input, boolean convertFirst, boolean lowercaseRest) {
        // -- create the result object
        StringBuffer result = new StringBuffer();
        
        // -- By setting this value we will be able to convert the first letter
        // -- too. This variable indicates if the next comming character should
        // -- be converted to uppercase.
        boolean nextInUppercase = convertFirst;
        
        for (char ch : input.toCharArray()) {
            // -- Check if we need to convert the next character to uppercase.
            // -- If we do we'll set the current character to it's uppercase
            // -- variant. Don't forget to disable the uppercase flag
            if (nextInUppercase) {
                ch = Character.toUpperCase(ch);
                nextInUppercase = false;
            } else {
                // -- If lowercaseRest is set, we'll try to convert every non
                // -- uppercase character to it's lowercase variant. This will
                // -- create nice camelcase strings, even with capitalized ones
                if (lowercaseRest)
                    ch = Character.toLowerCase(ch);
            }
            
            // -- if the character is an '_', we'll have to convert the next
            // -- character to uppercase. We won't add this '_' character to the
            // -- result
            if (ch == '_')
                nextInUppercase = true;
            
            // -- if it wasn't an '_', we'll just add the character to the 
            // -- result
            else
                result.append(ch);
        }
        
        // -- return the result
        return result.toString();
    }
    
// -- Testers -----------------------------------------------------------------
    
    public static boolean isInUppercase(String toCheck) {
        for (char ch : toCheck.toCharArray()) {
            if (Character.isLowerCase(ch))
                return false;
        }
        
        return true;
    }
    
    public static boolean isInLowercase(String toCheck) {
        for (char ch : toCheck.toCharArray()) {
            if (Character.isUpperCase(ch))
                return false;
        }
        
        return true;
    }
    
}
